// this is a very basic template. use it to start writing your own effects.
// if you want effects with lighting start from one of the GouraudXXXX or PhongXXXX effects

// --------------------------------------------------------------------------------------------------
// PARAMETERS:
// --------------------------------------------------------------------------------------------------

//transforms
float4x4 tW: WORLD;        //the models world matrix
float4x4 tV: VIEW;         //view matrix as set via Renderer (EX9)
float4x4 tP: PROJECTION;
float4x4 tWVP: WORLDVIEWPROJECTION;

//texture
texture Tex <string uiname="Texture";>;
sampler Samp = sampler_state    //sampler for doing the texture-lookup
{
    Texture   = (Tex);          //apply a texture to the sampler
    MipFilter = LINEAR;         //sampler states
    MinFilter = LINEAR;
    MagFilter = LINEAR;
};

//texture transformation marked with semantic TEXTUREMATRIX to achieve symmetric transformations
float4x4 tTex: TEXTUREMATRIX <string uiname="Texture Transform";>;

//pin for pixeldiameter and the offset of kernel filter
float factorPixelOffset <string uiname="Offset";> = 2;
int TexWidth <string uiname="Texture Width";> = 512;
int TexHeight <string uiname="Texture Heihgt";> = 512;
//the data structure: "vertexshader to pixelshader"
//used as output data with the VS function
//and as input data with the PS function
struct vs2ps
{
    float4 Pos  : POSITION;
    float2 TexCd : TEXCOORD0;
    float2 Offset : TEXCOORD1;
};

// --------------------------------------------------------------------------------------------------
// VERTEXSHADERS
// --------------------------------------------------------------------------------------------------
vs2ps VS(
    float4 Pos : POSITION,
    float4 TexCd : TEXCOORD0)
{
    //inititalize all fields of output struct with 0
    vs2ps Out = (vs2ps)0;

    //transform position
    Out.Pos = mul(Pos, tWVP);

    //transform texturecoordinates
    Out.TexCd = mul(TexCd, tTex);
    //factor 1 = pixeldiameter
    Out.Offset.x = 1*factorPixelOffset /float (TexWidth);
    Out.Offset.y = 1*factorPixelOffset /float (TexHeight);
    return Out;
}
// --------------------------------------------------------------------------------------------------
// PIXELSHADERS:
// --------------------------------------------------------------------------------------------------

float4 PS(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, In.TexCd);
    return col;
}



float FindMedian(float a, float b, float c)
{
float median;
             if( a < b )
                   {
                    if( b < c)
                        {
                        median = b;
                        }
                    else
                    {
                    median = max(a,c);
                    }
             }
             else
                    {
                    if( a < c)
                    {
                    median = a;
                    }
                    else
                    {
                    median = max(b,c);
             }
}

return median;
}

float4 PS3(vs2ps In): COLOR
{
    float4 resultY1 = tex2D(Samp, In.TexCd);
    float4 resultY2 = tex2D(Samp, In.TexCd);
    float4 resultY3 = tex2D(Samp, In.TexCd);
    float4 resultX = tex2D(Samp, In.TexCd);
    float3 texSamples[9];
    float CoordoX[9] = {-1, 0, 1, -1, 0, 1, -1, 0, 1,};
    float CoordoY[9] = {1, 1, 1, 0, 0, 0, -1, -1, -1,};

    for(int i=0; i <9; i++)
       { texSamples[i].xyz= tex2D( Samp, float2(In.TexCd.x+(In.Offset.x*CoordoX[i]) , In.TexCd.y+(In.Offset.y*CoordoY[i])));
}
resultY1.r = FindMedian(texSamples[0].r,texSamples[1].r,texSamples[2].r);
resultY1.g = FindMedian(texSamples[0].g,texSamples[1].g,texSamples[2].g);
resultY1.b = FindMedian(texSamples[0].b,texSamples[1].b,texSamples[2].b);

resultY2.r = FindMedian(texSamples[3].r,texSamples[4].r,texSamples[5].r);
resultY2.g = FindMedian(texSamples[3].g,texSamples[4].g,texSamples[5].g);
resultY2.b = FindMedian(texSamples[3].b,texSamples[4].b,texSamples[5].b);

resultY3.r = FindMedian(texSamples[6].r,texSamples[7].r,texSamples[8].r);
resultY3.g = FindMedian(texSamples[6].g,texSamples[7].g,texSamples[8].g);
resultY3.b = FindMedian(texSamples[6].b,texSamples[7].b,texSamples[8].b);


resultX.r = FindMedian(resultY1.r,resultY2.r,resultY3.r);
resultX.g = FindMedian(resultY1.g,resultY2.g,resultY3.g);
resultX.b = FindMedian(resultY1.b,resultY2.b,resultY3.b);

    return resultX;
}




// bypass
float4 PS0(vs2ps In): COLOR
{
    float4 col = tex2D(Samp, float2 (In.TexCd.x, In.TexCd.y));
    return col;
}

// --------------------------------------------------------------------------------------------------
// TECHNIQUES:
// --------------------------------------------------------------------------------------------------

technique Median
{
    pass P0
    {
        //Wrap0 = U;  // useful when mesh is round like a sphere
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_3_0 PS3();
    }
}



technique Bypass
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS0();
    }
}
